package com.ruoyi.business.controller;

import java.text.ParseException;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.business.domain.bo.AddApplyPlanBO;
import com.ruoyi.business.domain.bo.JobQueryBO;
import com.ruoyi.business.domain.bo.YinDaoCallBackBO;
import com.ruoyi.common.utils.bean.BeanUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.business.domain.ApplyPlan;
import com.ruoyi.business.service.IApplyPlanService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 应用执行计划管理Controller
 *
 * @author LiuChengRan
 * @date 2024-06-14
 */
@Api(tags = "应用执行计划管理接口")
@RestController
@RequestMapping("/business/ctApplyPlan")
@Slf4j
public class ApplyPlanController extends BaseController {
    @Autowired
    private IApplyPlanService applyPlanService;


    @PostMapping("/callBack/updatePlanStatus")
    public void updatePlanStatus(@RequestParam(required = false) String bodyMd5, @RequestParam(required = false) Long timestamp,
                                 @RequestParam(required = false) String sign, @RequestBody String body) {
        log.info("影刀回调成功：bodyMd5：{},sign:{},timestamp:{},body:{}", bodyMd5,sign,timestamp,body);

        YinDaoCallBackBO yinDaoCallBackBO = JSONObject.parseObject(body, YinDaoCallBackBO.class);
        // 获取应用名称 task_uuid
        if (applyPlanService.ydCallBack(yinDaoCallBackBO)) {
            log.debug("任务id:" + yinDaoCallBackBO.getJobUuid() + "回调成功，回调状态：" + yinDaoCallBackBO.getStatus());
        }else{
            log.debug("任务id:" + yinDaoCallBackBO.getJobUuid() + "回调失败，回调状态：" + yinDaoCallBackBO.getStatus());
        }
        //异步处理业务
        // new Runable() -> {
        // do things
        // }
    }

    /**
     * 修改运行时间
     */
    @ApiOperation(value = "修改运行时间", httpMethod = "POST", response = ApplyPlan.class)
    @PostMapping("/updateRunTime")
    public AjaxResult updateRunTime(@RequestBody ApplyPlan applyPlan) throws ParseException {
        return toAjax(applyPlanService.updateRunTime(applyPlan));
    }

    /**
     * 重新运行
     */
    @ApiOperation(value = "启用或禁用", httpMethod = "POST", response = ApplyPlan.class)
    @PostMapping("/stopOrStart")
    public AjaxResult appRetry(@RequestBody ApplyPlan applyPlan) throws IllegalAccessException {
        return toAjax(applyPlanService.updateById(applyPlan));
    }

    /**
     * 重新运行
     */
    @ApiOperation(value = "重新运行", httpMethod = "POST", response = ApplyPlan.class)
    @PostMapping("/appRetry")
    public AjaxResult appRetry(@RequestBody JobQueryBO jobQueryBO) throws IllegalAccessException {
        return toAjax(applyPlanService.appRetry(jobQueryBO));
    }

    /**
     * 停止计划运行
     */
    @ApiOperation(value = "停止计划运行", httpMethod = "POST", response = ApplyPlan.class)
    @PostMapping("/stopRun")
    public AjaxResult stopRun(@RequestBody JobQueryBO jobQueryBO) throws IllegalAccessException {
        return toAjax(applyPlanService.stopRun(jobQueryBO));
    }

    /**
     * 查询应用执行计划管理列表
     */
    @ApiOperation(value = "查询应用执行计划管理列表", httpMethod = "GET", response = ApplyPlan.class)
    @GetMapping("/list")
    public TableDataInfo list(ApplyPlan applyPlan) {
        startPage();
        List<ApplyPlan> list = applyPlanService.list(applyPlan);
        return getDataTable(list);
    }

    /**
     * 导出应用执行计划管理列表
     */
    @ApiOperation(value = "导出应用执行计划管理列表", httpMethod = "POST")
    @Log(title = "应用执行计划管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ApplyPlan applyPlan) {
        List<ApplyPlan> list = applyPlanService.list(applyPlan);
        ExcelUtil<ApplyPlan> util = new ExcelUtil<ApplyPlan>(ApplyPlan.class);
        util.exportExcel(response, list, "应用执行计划管理数据");
    }

    /**
     * 获取应用执行计划管理详细信息
     */
    @ApiOperation(value = "获取应用执行计划管理详细信息", httpMethod = "GET", response = ApplyPlan.class)
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        ApplyPlan applyPlan = applyPlanService.getById(id);
        return success(applyPlan);
    }

    /**
     * 新增应用执行计划管理
     */
    @ApiOperation(value = "新增应用执行计划管理", httpMethod = "POST")
    @Log(title = "应用执行计划管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AddApplyPlanBO addApplyPlanBO) throws ParseException {
        // 随机分配
        addApplyPlanBO.setPlanType("1");
        addApplyPlanBO.setWaitTimeout("10m");
        addApplyPlanBO.setPriority(0);
        return toAjax(applyPlanService.save(addApplyPlanBO));
    }

    /**
     * 修改应用执行计划管理
     */
    @ApiOperation(value = "修改应用执行计划管理", httpMethod = "PUT")
    @Log(title = "应用执行计划管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AddApplyPlanBO addApplyPlanBO) {
        return toAjax(applyPlanService.update(addApplyPlanBO));
    }

    /**
     * 删除应用执行计划管理
     */
    @ApiOperation(value = "删除应用执行计划管理", httpMethod = "DELETE")
    @Log(title = "应用执行计划管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(applyPlanService.removeByIds(Arrays.asList(ids)));
    }
}
