package com.ruoyi.business.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.business.domain.ApplyPlanLog;
import com.ruoyi.business.service.IApplyPlanLogService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 应用执行结果记录管理Controller
 *
 * @author LiuChengRan
 * @date 2024-06-14
 */
@Api(tags = "应用执行结果记录管理接口")
@RestController
@RequestMapping("/business/ctApplyPlanLog")
public class ApplyPlanLogController extends BaseController
{
    @Autowired
    private IApplyPlanLogService applyPlanLogService;

    /**
     * 根据计划查询所有执行日志
     */
    @ApiOperation(value="根据计划查询所有执行日志", httpMethod = "GET", response = ApplyPlanLog.class)
    @GetMapping("/listAll")
    public AjaxResult listAll(String planId)
    {
        ApplyPlanLog applyPlanLog = new ApplyPlanLog();
        applyPlanLog.setPlanId(planId);
        List<ApplyPlanLog> list = applyPlanLogService.list(applyPlanLog);
        return AjaxResult.success(list);
    }

    /**
     * 查询应用执行结果记录管理列表
     */
    @ApiOperation(value="查询应用执行结果记录管理列表", httpMethod = "GET", response = ApplyPlanLog.class)
    @GetMapping("/list")
    public TableDataInfo list(ApplyPlanLog applyPlanLog)
    {
        startPage();
        List<ApplyPlanLog> list = applyPlanLogService.list(applyPlanLog);
        return getDataTable(list);
    }

    /**
     * 导出应用执行结果记录管理列表
     */
    @ApiOperation(value="导出应用执行结果记录管理列表", httpMethod = "POST")
    @Log(title = "应用执行结果记录管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ApplyPlanLog applyPlanLog)
    {
        List<ApplyPlanLog> list = applyPlanLogService.list(applyPlanLog);
        ExcelUtil<ApplyPlanLog> util = new ExcelUtil<ApplyPlanLog>(ApplyPlanLog.class);
        util.exportExcel(response, list, "应用执行结果记录管理数据");
    }

    /**
     * 获取应用执行结果记录管理详细信息
     */
    @ApiOperation(value="获取应用执行结果记录管理详细信息", httpMethod = "GET", response = ApplyPlanLog.class)
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(applyPlanLogService.getById(id));
    }

    /**
     * 新增应用执行结果记录管理
     */
    @ApiOperation(value="新增应用执行结果记录管理", httpMethod = "POST")
    @Log(title = "应用执行结果记录管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody ApplyPlanLog applyPlanLog)
    {
        return toAjax(applyPlanLogService.save(applyPlanLog));
    }

    /**
     * 修改应用执行结果记录管理
     */
    @ApiOperation(value="修改应用执行结果记录管理", httpMethod = "PUT")
    @Log(title = "应用执行结果记录管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody ApplyPlanLog applyPlanLog)
    {
        return toAjax(applyPlanLogService.updateById(applyPlanLog));
    }

    /**
     * 删除应用执行结果记录管理
     */
    @ApiOperation(value="删除应用执行结果记录管理", httpMethod = "DELETE")
    @Log(title = "应用执行结果记录管理", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(applyPlanLogService.removeByIds(Arrays.asList(ids)));
    }
}
