package com.ruoyi.business.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.stream.Collectors;

import com.ruoyi.business.domain.Apply;
import com.ruoyi.business.domain.bo.AddBatchResourceLibraryBO;
import com.ruoyi.business.service.IApplyService;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.business.domain.ResourceLibrary;
import com.ruoyi.business.service.IResourceLibraryService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 资源库管理Controller
 *
 * @author LiuChengRan
 * @date 2024-06-17
 */
@Api(tags = "资源库管理接口")
@RestController
@RequestMapping("/business/ctResourceLibrary")
public class ResourceLibraryController extends BaseController {
    @Autowired
    private IResourceLibraryService resourceLibraryService;

    @Autowired
    private IApplyService applyService;

    /**
     * 根据应用查询所需参数
     */
    @ApiOperation(value = "根据应用查询所需参数", httpMethod = "GET", response = ResourceLibrary.class)
    @GetMapping("/listByAppId")
    public AjaxResult listByAppId(String appId) {
        List<ResourceLibrary> list = resourceLibraryService.lambdaQuery().eq(ResourceLibrary::getAppId, appId).list();
        return AjaxResult.success(list);
    }

    /**
     * 查询资源库管理列表
     */
    @ApiOperation(value = "查询资源库管理列表", httpMethod = "GET", response = ResourceLibrary.class)
    @GetMapping("/list")
    public TableDataInfo list(ResourceLibrary resourceLibrary) {
        // 是否管理员
        SysUser user = SecurityUtils.getLoginUser().getUser();
        boolean isRole = user.getRoles().stream().filter(e -> e.getRoleId() == 1).count() > 0;

        if (!isRole) {
            List<String> appIdList = applyService.lambdaQuery().eq(Apply::getDeptId, SecurityUtils.getDeptId()).list().stream().map(Apply::getAppId).collect(Collectors.toList());
            startPage();
            List<ResourceLibrary> list = resourceLibraryService.list(resourceLibrary, appIdList);
            return getDataTable(list);
        } else {
            startPage();
            List<ResourceLibrary> list = resourceLibraryService.list(resourceLibrary, null);
            return getDataTable(list);
        }
    }

    /**
     * 导出资源库管理列表
     */
    @ApiOperation(value = "导出资源库管理列表", httpMethod = "POST")
    @Log(title = "资源库管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ResourceLibrary resourceLibrary) {
        List<String> appIdList = applyService.lambdaQuery().eq(Apply::getDeptId, SecurityUtils.getDeptId()).list().stream().map(Apply::getAppId).collect(Collectors.toList());
        // 是否管理员
        boolean isRole = Arrays.stream(SecurityUtils.getLoginUser().getUser().getRoleIds()).filter(e -> e == 1).count() > 0;
        if (!isRole) {
            List<ResourceLibrary> list = resourceLibraryService.list(resourceLibrary, appIdList);
            ExcelUtil<ResourceLibrary> util = new ExcelUtil<ResourceLibrary>(ResourceLibrary.class);
            util.exportExcel(response, list, "资源库管理数据");
        } else {
            List<ResourceLibrary> list = resourceLibraryService.list(resourceLibrary, null);
            ExcelUtil<ResourceLibrary> util = new ExcelUtil<ResourceLibrary>(ResourceLibrary.class);
            util.exportExcel(response, list, "资源库管理数据");
        }

    }

    /**
     * 获取资源库管理详细信息
     */
    @ApiOperation(value = "获取资源库管理详细信息", httpMethod = "GET", response = ResourceLibrary.class)
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(resourceLibraryService.getById(id));
    }

    /**
     * 批量修改资源信息
     */
    @ApiOperation(value = "批量修改资源信息", httpMethod = "POST")
    @Log(title = "批量修改资源信息", businessType = BusinessType.INSERT)
    @PostMapping("/updateBatch")
    public AjaxResult updateBatch(@RequestBody AddBatchResourceLibraryBO addBatchResourceLibraryBO) {
        return toAjax(resourceLibraryService.updateBatch(addBatchResourceLibraryBO));
    }

    /**
     * 批量新增资源信息
     */
    @ApiOperation(value = "批量新增资源信息", httpMethod = "POST")
    @Log(title = "批量新增资源信息", businessType = BusinessType.INSERT)
    @PostMapping("/addBatch")
    public AjaxResult addBatch(@RequestBody AddBatchResourceLibraryBO addBatchResourceLibraryBO) {
        return toAjax(resourceLibraryService.addBatch(addBatchResourceLibraryBO));
    }


    /**
     * 新增资源库管理
     */
    @ApiOperation(value = "新增资源库管理", httpMethod = "POST")
    @Log(title = "资源库管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody ResourceLibrary resourceLibrary) {
        Integer count = resourceLibraryService.lambdaQuery().eq(ResourceLibrary::getAppId, resourceLibrary.getAppId()).eq(ResourceLibrary::getResourceName, resourceLibrary.getResourceName()).count();
        if (count > 0) {
            return AjaxResult.error("已经有相同的资源键，无需重复创建");
        }
        return toAjax(resourceLibraryService.save(resourceLibrary));
    }

    /**
     * 修改资源库管理
     */
    @ApiOperation(value = "修改资源库管理", httpMethod = "PUT")
    @Log(title = "资源库管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody ResourceLibrary resourceLibrary) {
        return toAjax(resourceLibraryService.updateById(resourceLibrary));
    }

    /**
     * 删除资源库管理
     */
    @ApiOperation(value = "删除资源库管理", httpMethod = "DELETE")
    @Log(title = "资源库管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(resourceLibraryService.removeByIds(Arrays.asList(ids)));
    }
}
