package com.ruoyi.business.service.impl;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.business.domain.*;
import com.ruoyi.business.domain.bo.*;
import com.ruoyi.business.domain.vo.JobQueryLogVO;
import com.ruoyi.business.domain.vo.JobQueryVO;
import com.ruoyi.business.domain.vo.JobStartVO;
import com.ruoyi.business.domain.vo.ListRebotVO;
import com.ruoyi.business.service.*;
import com.ruoyi.business.util.YinDaoHttpUtils;
import com.ruoyi.business.yddoman.BaseDTO;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.enums.ExcTypeStatus;
import com.ruoyi.common.enums.PlanRunStatus;
import com.ruoyi.common.enums.RebotStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import org.quartz.*;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.support.CronSequenceGenerator;
import org.springframework.stereotype.Service;
import com.ruoyi.business.mapper.ApplyPlanMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

/**
 * 应用执行计划管理Service业务层处理
 *
 * @author LiuChengRan
 * @date 2024-06-14
 */
@Service
public class ApplyPlanServiceImpl extends ServiceImpl<ApplyPlanMapper, ApplyPlan> implements IApplyPlanService {
    @Autowired
    private IApplyService applyService;

    @Autowired
    private IRebotService rebotService;

    @Autowired
    private IResourceLibraryService resourceLibraryService;

    @Autowired
    private RedissonClient redissonClient;

    @Autowired
    private IApplyPlanLogService applyPlanLogService;

    @Autowired
    private IApplyPlanService applyPlanService;

    /**
     * 修改运行时间
     *
     * @param entity
     * @return
     */
    @Override
    public boolean updateRunTime(ApplyPlan entity) throws ParseException {
        ApplyPlan applyPlan = this.getById(entity.getId());
        final CronExpression cron = new CronExpression(applyPlan.getCronExpression());
        // 下次执行时间
        Date nextDate = null;
        if (Objects.isNull(applyPlan.getLastExecTime())) {
            Date date = new Date();
            log.debug("当前时间:" + date);
            nextDate = cron.getNextValidTimeAfter(date);//下次执行时间
            log.debug("下次时间:" + nextDate);
            applyPlan.setNextExecTime(nextDate);
        } else {
            log.debug("当前时间:" + applyPlan.getNextExecTime());
            nextDate = cron.getNextValidTimeAfter(applyPlan.getLastExecTime());//下次执行时间
            log.debug("下次时间:" + nextDate);
            // 如果当前时间大于下次运行时间 证明已经超过计划执行时间，下次时间以当前时间为主
            Date date = new Date();
            if (date.compareTo(nextDate) > 0) {
                nextDate = cron.getNextValidTimeAfter(date);//下次执行时间
                applyPlan.setNextExecTime(nextDate);
            } else {
                nextDate = cron.getNextValidTimeAfter(applyPlan.getLastExecTime());//下次执行时间
                applyPlan.setNextExecTime(nextDate);
            }
        }
        return this.updateById(applyPlan);
    }

    /**
     * 影刀回调
     *
     * @param yinDaoCallBackBO
     * @return
     */
    @Override
    @Transactional
    public boolean ydCallBack(YinDaoCallBackBO yinDaoCallBackBO) {
        RLock lock = redissonClient.getLock("updateResult");
        try {
            // 尝试获取锁，最多等待100秒，锁定之后最多持有锁10秒
            boolean isLocked = lock.tryLock(2, 10, TimeUnit.SECONDS);
            if (isLocked) {
                ApplyPlan applyPlan = this.lambdaQuery().eq(ApplyPlan::getTaskUuid, yinDaoCallBackBO.getJobUuid()).one();
                applyPlan.setStartTime(DateUtils.parseDate(yinDaoCallBackBO.getStartTime()));
                applyPlan.setEndTime(DateUtils.parseDate(yinDaoCallBackBO.getEndTime()));
                if (null != yinDaoCallBackBO.getEndTime()) {
                    log.debug("endTime：" + yinDaoCallBackBO.getEndTime());
                    Date endDate = DateUtils.parseDate(yinDaoCallBackBO.getEndTime());
                    Date startDate = DateUtils.parseDate(yinDaoCallBackBO.getStartTime());
                    BigDecimal planTime = new BigDecimal((double) (endDate.getTime() - startDate.getTime()) / (60 * 1000)).setScale(2, RoundingMode.HALF_UP);
                    applyPlan.setPlanTime(planTime + "");
                }
                applyPlan.setTaskStatus(yinDaoCallBackBO.getStatus());

                if (!yinDaoCallBackBO.getResult().isEmpty()) {
                    applyPlan.setOutParam(yinDaoCallBackBO.getResult().get(0).getValue());

                    // 记录响应参数
                    ApplyPlanLog applyPlanLog = applyPlanLogService.lambdaQuery().eq(ApplyPlanLog::getJobUuid, yinDaoCallBackBO.getJobUuid()).one();
                    if (null == applyPlanLog) {
                        applyPlanLog = new ApplyPlanLog();
                        applyPlanLog.setCreateBy("系统创建");
                        applyPlanLog.setCreateTime(new Date());
                    } else {
                        applyPlanLog.setUpdateBy("系统创建");
                        applyPlanLog.setUpdateTime(new Date());
                    }
                    applyPlanLog.setJobUuid(yinDaoCallBackBO.getJobUuid());
                    applyPlanLog.setAppId(applyPlan.getAppId());
                    applyPlanLog.setOutParam(applyPlan.getOutParam());
                    applyPlanLogService.saveOrUpdate(applyPlanLog);
                }
                applyPlan.setUpdateBy("系统修改");
                applyPlan.setUpdateTime(new Date());
                boolean isUpdate = this.updateById(applyPlan);
                // 终态
                List<String> finalState = new ArrayList<>();
                finalState.add(PlanRunStatus.FINISH.getKey());
                finalState.add(PlanRunStatus.STOPPED.getKey());
                finalState.add(PlanRunStatus.ERROR.getKey());
                finalState.add(PlanRunStatus.SKIPPED.getKey());
                finalState.add(PlanRunStatus.CANCEL.getKey());
                // 如果是终态
                if (StringUtils.isNotEmpty(yinDaoCallBackBO.getRobotClientName()) && finalState.contains(yinDaoCallBackBO.getStatus())) {
                    Rebot rebot = new Rebot();
                    rebot.setStatus(RebotStatus.IDLE.getKey());
                    rebot.setUpdateBy("系统修改");
                    rebot.setUpdateTime(new Date());
                    return rebotService.lambdaUpdate().eq(Rebot::getRobotClientName, yinDaoCallBackBO.getRobotClientName()).update(rebot);
                } else {
                    return isUpdate;
                }
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            if (lock.isHeldByCurrentThread()) {
                // 释放锁
                lock.unlock();
            }
        }
        return true;
    }

    /**
     * 重新运行
     *
     * @param jobQueryBO
     * @return
     * @throws IllegalAccessException
     */
    @Override
    public boolean appRetry(JobQueryBO jobQueryBO) throws IllegalAccessException {
        BaseDTO baseDTO = YinDaoHttpUtils.appRetry(jobQueryBO);
        if (baseDTO.getSuccess()) {
            return baseMapper.appRetry(jobQueryBO.getJobUuid()) > 0;
        } else {
            return false;
        }
    }

    /**
     * 停止计划运行
     *
     * @return
     */
    @Override
    public boolean stopRun(JobQueryBO jobQueryBO) throws IllegalAccessException {
        BaseDTO baseDTO = YinDaoHttpUtils.appStop(jobQueryBO);
        return baseDTO.getSuccess();
    }

    /**
     * 指定时间执行
     *
     * @return
     */
    @Override
    @Transactional
    public boolean runAppBySetTime() throws IllegalAccessException {
        RLock lock = redissonClient.getLock("runAppBySetTime");
        try {
            // 尝试获取锁，最多等待100秒，锁定之后最多持有锁10秒
            boolean isLocked = lock.tryLock(2, 10, TimeUnit.SECONDS);
            if (isLocked) {
                List<ApplyPlan> applyPlanList = this.lambdaQuery().eq(ApplyPlan::getExcType, ExcTypeStatus.TWO.getKey())
                        .isNull(ApplyPlan::getTaskUuid).orderByAsc(ApplyPlan::getExcTime)
                        .eq(ApplyPlan::getEnabled, "0")
                        .list();
                if (applyPlanList.isEmpty()) {
                    return false;
                }

                ListRebotBO listRebotBO = new ListRebotBO();
                listRebotBO.setStatus(RebotStatus.IDLE.getKey());
                listRebotBO.setPage(1);
                listRebotBO.setSize(100);
                // 只获取空闲的机器人
                List<ListRebotVO> listRebotVos = YinDaoHttpUtils.listRebot(listRebotBO);
                if (listRebotVos.isEmpty()) {
                    log.debug("暂无空闲机器人");
                    // 刷新机器人数据
//            rebotService.syn(new Rebot());
                    return false;
                }
                List<String> ids = new ArrayList<>();
                listRebotVos.forEach(rebot -> {
                    applyPlanList.forEach(e -> {
                        log.debug("指定时间执行 执行时间：" + e.getExcTime());
                        if (ids.contains(e.getId())) {
                            return;
                        }
                        if (e.getExcTime().getTime() <= System.currentTimeMillis() && PlanRunStatus.AWAIT_CREATE.getKey().equals(e.getTaskStatus())) {
                            ApplyStartBO applyStartBO = new ApplyStartBO();
                            applyStartBO.setRobotUuid(e.getAppId());
                            applyStartBO.setAccountName(rebot.getRobotClientName());
                            applyStartBO.setManualTime(e.getManualTime());
                            //high 高 middle 中 low 低
                            applyStartBO.setPriority("high");
                            // 如果应用支持参数
                            if (!Objects.isNull(e.getSupportParam()) && e.getSupportParam() == 1) {
                                if (StringUtils.isNotEmpty(e.getPlanParams())) {
                                    applyStartBO.setPlanParams(e.getPlanParams());
                                    List<ApplyStartBO.RobotParam> paramList = JSON.parseArray(e.getPlanParams(), ApplyStartBO.RobotParam.class);
                                    applyStartBO.setParams(paramList);
                                }
                            }
                            try {
                                JobStartVO jobStartVO = YinDaoHttpUtils.appStart(applyStartBO);
                                ApplyPlan applyPlan = new ApplyPlan();
                                applyPlan.setId(e.getId());
                                applyPlan.setUpdateBy("系统修改");
                                applyPlan.setUpdateTime(new Date());
                                applyPlan.setTaskUuid(jobStartVO.getJobUuid());
                                applyPlan.setTaskStatus(PlanRunStatus.CREATED.getKey());
                                this.updateById(applyPlan);
                                Rebot updateRebot = new Rebot();
                                updateRebot.setUpdateBy("系统修改");
                                updateRebot.setUpdateTime(new Date());
                                updateRebot.setStatus(RebotStatus.RUNNING.getKey());
                                if (rebotService.lambdaUpdate().eq(Rebot::getRobotClientName, rebot.getRobotClientName()).update(updateRebot)) {
                                    log.debug("applyPlanList:" + applyPlanList.size());
                                    ids.add(e.getId());
                                }
                            } catch (IllegalAccessException ex) {
                                throw new RuntimeException(ex);
                            }
                        }
                    });
                });
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            if (lock.isHeldByCurrentThread()) {
                // 释放锁
                lock.unlock();
            }
        }
        return false;
    }

    /**
     * 表达式执行
     */
    @Override
    public boolean runAppByCron() throws IllegalAccessException {
        RLock lock = redissonClient.getLock("runAppByCron");
        try {
            // 尝试获取锁，最多等待100秒，锁定之后最多持有锁10秒
            boolean isLocked = lock.tryLock(2, 10, TimeUnit.SECONDS);
            if (isLocked) {
                // 先查计划，避免每次命中第三方接口
                List<ApplyPlan> applyPlanList = this.lambdaQuery().eq(ApplyPlan::getExcType, ExcTypeStatus.TREE.getKey())
                        .ne(ApplyPlan::getTaskStatus, PlanRunStatus.RUNNING.getKey())
                        .ne(ApplyPlan::getTaskStatus, PlanRunStatus.CREATED.getKey())
                        .eq(ApplyPlan::getEnabled, "0")
                        .list();
                if (applyPlanList.isEmpty()) {
                    return false;
                }
                // 需要执行的计划
                List<ApplyPlan> execApplyPlan = new ArrayList<>();
                applyPlanList.forEach(e -> {
                    Date date = new Date();
                    // 当前时间大于下次时间
                    if (date.compareTo(e.getNextExecTime()) > -1) {
                        // 执行一次
                        execApplyPlan.add(e);
                    }
                });
                if (execApplyPlan.isEmpty()) {
                    log.debug("没有待执行的计划");
                    return false;
                }

                ListRebotBO listRebotBO = new ListRebotBO();
                listRebotBO.setStatus(RebotStatus.IDLE.getKey());
                listRebotBO.setPage(1);
                listRebotBO.setSize(100);
                // 只获取空闲的机器人
                List<ListRebotVO> listRebotVos = YinDaoHttpUtils.listRebot(listRebotBO);
                if (listRebotVos.isEmpty()) {
                    log.debug("暂无空闲机器人");
                    // 刷新机器人数据
//            rebotService.syn(new Rebot());
                    return false;
                }
                List<String> ids = new ArrayList<>();
                execApplyPlan.forEach(e -> {
                    if (listRebotVos.isEmpty()) {
                        // 刷新机器人数据
//            rebotService.syn(new Rebot());
                        return;
                    }
                    ListRebotVO rebot = listRebotVos.get(0);
                    log.debug("机器人表达式执行调度日志" + rebot.getRobotClientName() + "调度" + e.getAppName() + "应用,排除的应用id为" + ids.toString());
                    if (ids.contains(e.getId())) {
                        return;
                    }
                    log.debug("机器人表达式成功调度日志" + rebot.getRobotClientName() + "调度" + e.getAppName() + "应用,排除的应用id为" + ids.toString());
                    // 该计划满足执行条件
                    ApplyStartBO applyStartBO = new ApplyStartBO();
                    applyStartBO.setRobotUuid(e.getAppId());
                    applyStartBO.setAccountName(rebot.getRobotClientName());
                    applyStartBO.setManualTime(e.getManualTime());
                    //high 高 middle 中 low 低
                    applyStartBO.setPriority("high");

                    // 如果应用支持参数
                    if (!Objects.isNull(e.getSupportParam()) && e.getSupportParam() == 1) {
                        List<ResourceLibrary> list = resourceLibraryService.lambdaQuery().eq(ResourceLibrary::getAppId, e.getAppId()).list();
                        if (!list.isEmpty()) {
                            List<ApplyStartBO.RobotParam> robotParamList = list.stream().map(param -> {
                                ApplyStartBO.RobotParam robotParam = new ApplyStartBO.RobotParam();
                                robotParam.setType(param.getResourceType());
                                robotParam.setName(param.getResourceName());
                                robotParam.setValue(param.getResourceValue());
                                return robotParam;
                            }).collect(Collectors.toList());
                            applyStartBO.setParams(robotParamList);
                            // 记录已经执行的参数
                            String jsonString = JSON.toJSONString(robotParamList);
                            applyStartBO.setPlanParams(jsonString);
                        }
                    }

                    try {
                        JobStartVO jobStartVO = YinDaoHttpUtils.appStart(applyStartBO);
                        ApplyPlan applyPlan = new ApplyPlan();
                        applyPlan.setId(e.getId());
                        applyPlan.setUpdateBy("系统修改");
                        applyPlan.setUpdateTime(new Date());
                        applyPlan.setTaskUuid(jobStartVO.getJobUuid());
                        applyPlan.setTaskStatus(PlanRunStatus.RUNNING.getKey());
                        // 实际执行的参数
                        applyPlan.setPlanParams(applyStartBO.getPlanParams());
                        // 根据表达式返回下一个执行时间
                        log.debug("表达式：" + e.getCronExpression());
                        final CronExpression cron = new CronExpression(e.getCronExpression());
                        Date nextDate = null;
                        // 如果上次时间为空，下次执行时间也为空 下次时间以当前时间计算，上次时间为当前时间
                        if (Objects.isNull(e.getLastExecTime()) && Objects.isNull(e.getNextExecTime())) {
                            Date date = new Date();
                            log.debug("当前时间:" + date);
                            nextDate = cron.getNextValidTimeAfter(date);//下次执行时间
                            log.debug("下次时间:" + nextDate);
                            applyPlan.setLastExecTime(date);
                            applyPlan.setNextExecTime(nextDate);
                            // 上次时间记录为下次时间，下次时间更新为计划下次时间
                        } else {
                            applyPlan.setLastExecTime(e.getNextExecTime());
                            log.debug("当前时间:" + e.getNextExecTime());
                            nextDate = cron.getNextValidTimeAfter(e.getNextExecTime());//下次执行时间
                            log.debug("下次时间:" + nextDate);
                            applyPlan.setNextExecTime(nextDate);
                        }
                        this.updateById(applyPlan);
                        Rebot updateRebot = new Rebot();
                        updateRebot.setUpdateBy("系统修改");
                        updateRebot.setUpdateTime(new Date());
                        updateRebot.setStatus(RebotStatus.RUNNING.getKey());
                        if (rebotService.lambdaUpdate().eq(Rebot::getRobotClientName, rebot.getRobotClientName()).update(updateRebot)) {
                            ids.add(e.getId());
                        }
                        // 记录执行日志
                        ApplyPlanLog applyPlanLog = applyPlanLogService.lambdaQuery().eq(ApplyPlanLog::getJobUuid, jobStartVO.getJobUuid()).one();
                        if (null == applyPlanLog) {
                            applyPlanLog = new ApplyPlanLog();
                            applyPlanLog.setCreateBy("系统创建");
                            applyPlanLog.setCreateTime(new Date());
                        } else {
                            applyPlanLog.setUpdateBy("系统修改");
                            applyPlanLog.setUpdateTime(new Date());
                        }
                        applyPlanLog.setJobUuid(jobStartVO.getJobUuid());
                        applyPlanLog.setAppId(e.getAppId());
                        applyPlanLog.setPlanId(applyPlan.getId());
                        applyPlanLogService.saveOrUpdate(applyPlanLog);
                    } catch (IllegalAccessException ex) {
                        throw new RuntimeException(ex);
                    } catch (ParseException ex) {
                        throw new RuntimeException(ex);
                    }
                    listRebotVos.remove(0);
                });
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            if (lock.isHeldByCurrentThread()) {
                // 释放锁
                lock.unlock();
            }
        }
        return false;
    }

    /**
     * 获取应用执行结果
     *
     * @return
     * @throws IllegalAccessException
     */
    @Override
    @Transactional
    public void queryAppStartResult() throws IllegalAccessException {
        RLock lock = redissonClient.getLock("updateResult");
        try {
            // 尝试获取锁，最多等待100秒，锁定之后最多持有锁10秒
            boolean isLocked = lock.tryLock(2, 10, TimeUnit.SECONDS);
            if (isLocked) {
                // 不查询的状态
                List<String> noStatus = new ArrayList<>();
                noStatus.add(PlanRunStatus.FINISH.getKey());
                noStatus.add(PlanRunStatus.STOPPED.getKey());
                noStatus.add(PlanRunStatus.ERROR.getKey());
                noStatus.add(PlanRunStatus.SKIPPED.getKey());
                noStatus.add(PlanRunStatus.CANCEL.getKey());
                // 查询任务运行状态非空非终态并且已经创建的任务
                List<ApplyPlan> list = this.lambdaQuery().notIn(ApplyPlan::getTaskStatus, noStatus).isNotNull(ApplyPlan::getTaskUuid).isNotNull(ApplyPlan::getTaskStatus).orderByAsc(ApplyPlan::getPriority).list();
                if (list.isEmpty()) {
                    log.debug("没有需要更新状态的计划任务");
                    return;
                }
                for (ApplyPlan applyPlan : list) {
                    JobQueryBO jobQueryBO = new JobQueryBO();
                    jobQueryBO.setJobUuid(applyPlan.getTaskUuid());
                    JobQueryVO jobQueryVO = YinDaoHttpUtils.queryAppStartResult(jobQueryBO);
                    // 如果状态不一致，更新最新状态
                    if (jobQueryVO != null && !applyPlan.getTaskStatus().equals(jobQueryVO.getStatus())) {
                        // 修改的任务计划
                        ApplyPlan updateApplyPlan = new ApplyPlan();
                        updateApplyPlan.setTaskStatus(jobQueryVO.getStatus());
                        if (null != jobQueryVO.getEndTime()) {
                            updateApplyPlan.setEndTime(jobQueryVO.getEndTime());
                            updateApplyPlan.setStartTime(jobQueryVO.getStartTime());
                            if (!Objects.isNull(updateApplyPlan.getEndTime()) && !Objects.isNull(updateApplyPlan.getStartTime())) {
                                long diff = updateApplyPlan.getEndTime().getTime() - updateApplyPlan.getStartTime().getTime();
                                BigDecimal planTime = new BigDecimal((double) diff / (60 * 1000)).setScale(2, RoundingMode.HALF_UP);
                                updateApplyPlan.setPlanTime(planTime + "");
                                if (!Objects.isNull(applyPlan.getManualTime())) {
                                    updateApplyPlan.setTimeSaving((new BigDecimal(applyPlan.getManualTime()).subtract(planTime)) + "");
                                }
                            }
                        }
                        updateApplyPlan.setId(applyPlan.getId());
                        updateApplyPlan.setUpdateBy("系统修改");
                        updateApplyPlan.setUpdateTime(new Date());

                        // 如果是终态并且有响应参数 获取返回值
                        List<String> finishStatus = Arrays.asList(PlanRunStatus.FINISH.getKey(), PlanRunStatus.STOPPED.getKey(), PlanRunStatus.ERROR.getKey(), PlanRunStatus.SKIPPED.getKey(), PlanRunStatus.CANCEL.getKey());
                        if (finishStatus.contains(jobQueryVO.getStatus())) {
                            List<JobQueryVO.RobotParam> outputs = jobQueryVO.getRobotParams().getOutputs();
                            if (null != outputs && !outputs.isEmpty()) {
                                JobQueryVO.RobotParam robotParam = outputs.get(0);
                                updateApplyPlan.setOutParam(robotParam.getValue());
                            }
                        }
                        updateApplyPlan.setRemark(jobQueryVO.getRemark());

                        this.updateById(updateApplyPlan);
                        if ("2".equals(applyPlan.getExcType())) {
                            ApplyPlanLog applyPlanLog = applyPlanLogService.lambdaQuery().eq(ApplyPlanLog::getJobUuid, applyPlan.getTaskUuid()).one();
                            // 记录响应参数
                            if (applyPlanLog == null) {
                                applyPlanLog = new ApplyPlanLog();
                                applyPlanLog.setCreateBy("系统创建");
                                applyPlanLog.setCreateTime(new Date());
                            } else {
                                applyPlanLog.setUpdateBy("系统创建");
                                applyPlanLog.setUpdateTime(new Date());
                            }
                            applyPlanLog.setJobUuid(jobQueryVO.getJobUuid());
                            applyPlanLog.setAppId(applyPlan.getAppId());
                            applyPlanLog.setOutParam(updateApplyPlan.getOutParam());
                            applyPlanLogService.saveOrUpdate(applyPlanLog);
                        }
                    }
                    // 如果是终态改变机器人状态
                    List<String> finishStatus = Arrays.asList(PlanRunStatus.FINISH.getKey(), PlanRunStatus.STOPPED.getKey(), PlanRunStatus.ERROR.getKey(), PlanRunStatus.SKIPPED.getKey(), PlanRunStatus.CANCEL.getKey());
                    if (finishStatus.contains(jobQueryVO.getStatus())) {
                        Rebot rebot = new Rebot();
                        rebot.setStatus(RebotStatus.IDLE.getKey());
                        rebot.setUpdateBy("系统修改");
                        rebot.setUpdateTime(new Date());
                        rebot.setStatus(RebotStatus.IDLE.getKey());
                        rebotService.lambdaUpdate().eq(Rebot::getRobotClientName, jobQueryVO.getRobotClientName()).update(rebot);
                    } else if (PlanRunStatus.RUNNING.getKey().equals(jobQueryVO.getStatus())) {
                        Rebot rebot = new Rebot();
                        rebot.setStatus(RebotStatus.RUNNING.getKey());
                        rebot.setUpdateBy("系统修改");
                        rebot.setUpdateTime(new Date());
                        rebotService.lambdaUpdate().eq(Rebot::getRobotClientName, jobQueryVO.getRobotClientName()).update(rebot);
                    }
                }
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            if (lock.isHeldByCurrentThread()) {
                // 释放锁
                lock.unlock();
            }
        }
    }

    /**
     * 执行计划（针对立即执行的计划）
     *
     * @param applyStartBO 该对象只有 应用id 优先级和人工用时
     * @return
     */
    @Override
    @Transactional
    public boolean execPlan(ApplyStartBO applyStartBO, ApplyPlan applyPlan) throws IllegalAccessException {
        RLock lock = redissonClient.getLock("execPlan");
        int i = 0;
        try {
            // 尝试获取锁，最多等待100秒，锁定之后最多持有锁10秒
            boolean isLocked = lock.tryLock(2, 20, TimeUnit.SECONDS);
            if (isLocked) {
                // 先查计划 避免每次直接命中第三方接口
                Map<Long, List<ApplyPlan>> applyPlanMap = this.lambdaQuery().eq(ApplyPlan::getExcType, ExcTypeStatus.ONE.getKey()).isNull(ApplyPlan::getTaskUuid)
                        .orderByAsc(ApplyPlan::getPriority).list().stream().collect(Collectors.groupingBy(ApplyPlan::getDeptId));

                // 查询所有待执行的任务
                if (applyPlanMap.isEmpty()) {
                    log.debug("没有等待执行的计划");
                    return false;
                }

                ListRebotBO listRebotBO = new ListRebotBO();
                listRebotBO.setStatus(RebotStatus.IDLE.getKey());
                listRebotBO.setPage(1);
                listRebotBO.setSize(100);
                // 只获取空闲的机器人
                List<ListRebotVO> listRebotVos = YinDaoHttpUtils.listRebot(listRebotBO);
                if (listRebotVos.isEmpty()) {
                    log.debug("暂无空闲机器人");
                    return false;
                }

                // 获取每个部门优先级最高的计划
                List<ApplyPlan> applyPlans = new ArrayList<>();
                applyPlanMap.forEach((k, v) -> {
                    v.sort((Comparator.comparingInt(ApplyPlan::getPriority)));
                    applyPlans.add(v.get(0));
                });
                if (applyPlans.isEmpty()) {
                    log.debug("没有等待执行的计划");
                    return false;
                }
                applyPlans.sort((Comparator.comparingInt(ApplyPlan::getPriority)));

                for (ListRebotVO listRebotVO : listRebotVos) {
                    if (applyPlans.isEmpty()) {
                        log.debug("没有等待执行的计划！");
                        continue;
                    }
                    applyPlan = applyPlans.get(0);
                    applyStartBO = new ApplyStartBO();
                    applyStartBO.setRobotUuid(applyPlan.getAppId());
                    //high 高 middle 中 low 低
                    applyStartBO.setPriority("high");
                    applyStartBO.setAccountName(listRebotVO.getRobotClientName());
                    // 如果应用支持参数
                    if (!Objects.isNull(applyPlan.getSupportParam()) && applyPlan.getSupportParam() == 1) {
                        List<ResourceLibrary> list = resourceLibraryService.lambdaQuery().eq(ResourceLibrary::getAppId, applyPlan.getAppId()).list();
                        if (!list.isEmpty()) {
                            List<ApplyStartBO.RobotParam> robotParamList = list.stream().map(e -> {
                                ApplyStartBO.RobotParam robotParam = new ApplyStartBO.RobotParam();
                                robotParam.setType(e.getResourceType());
                                robotParam.setName(e.getResourceName());
                                robotParam.setValue(e.getResourceValue());
                                return robotParam;
                            }).collect(Collectors.toList());
                            applyStartBO.setParams(robotParamList);
                            // 记录已经执行的参数
                            String jsonString = JSON.toJSONString(robotParamList);
                            applyStartBO.setPlanParams(jsonString);
                        }
                    }
                    JobStartVO jobStartVO = YinDaoHttpUtils.appStart(applyStartBO);
                    ApplyPlan updateApplyPlan = new ApplyPlan();
                    updateApplyPlan.setId(applyPlan.getId());
                    updateApplyPlan.setCreateBy("系统创建");
                    updateApplyPlan.setCreateTime(new Date());
                    updateApplyPlan.setUpdateBy("系统修改");
                    updateApplyPlan.setUpdateTime(new Date());
                    updateApplyPlan.setStartTime(new Date());
                    updateApplyPlan.setTaskUuid(jobStartVO.getJobUuid());
                    updateApplyPlan.setPlanName(applyPlan.getPlanName());
                    updateApplyPlan.setTaskStatus(PlanRunStatus.CREATED.getKey());
                    updateApplyPlan.setRobotName(listRebotVO.getRobotClientName());
                    updateApplyPlan.setManualTime(applyPlan.getManualTime());
                    updateApplyPlan.setDeptId(applyStartBO.getDeptId());
                    updateApplyPlan.setPlanParams(applyStartBO.getPlanParams());
                    applyPlan.setTaskUuid(jobStartVO.getJobUuid());
                    if (this.saveOrUpdate(updateApplyPlan)) {
                        // 移除当前对象，防止重复跑
                        applyPlans.remove(0);
                        i++;
                    }
                }
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            if (lock.isHeldByCurrentThread()) {
                // 释放锁
                lock.unlock();
            }
        }
        return i > 0;
    }


    @Override
    @Transactional
    public boolean save(AddApplyPlanBO addApplyPlanBO) throws ParseException {
        // 指定时间执行
        if (addApplyPlanBO.getExcType().equals("1") && null == addApplyPlanBO.getExcTime()) {
            throw new ServiceException("指定时间执行时间不能为空");
        }
        // 表达式执行
        if (addApplyPlanBO.getExcType().equals("2")) {
            if (null == addApplyPlanBO.getCronExpression()) {
                throw new ServiceException("周期执行表达式不能为空");
            }
            try {
                // cron 验证表达式是否正确
                CronExpression.validateExpression(addApplyPlanBO.getCronExpression());
            } catch (Exception ex) {
                throw new ServiceException("时间表达式错误");
            }
        }

        List<ApplyPlan> applyPlanList = new ArrayList<>();
        Map<String, List<Apply>> appMap = applyService.lambdaQuery().in(Apply::getAppId, addApplyPlanBO.getAppId()).list().stream().collect(Collectors.groupingBy(Apply::getAppId));
        // 是否有空闲机器人并且创建计划成功
        boolean isExec = false;
        AtomicBoolean insert = new AtomicBoolean(false);

        ApplyPlan applyPlan = new ApplyPlan();
        if ("2".equals(addApplyPlanBO.getExcType())) {
            final CronExpression cron = new CronExpression(addApplyPlanBO.getCronExpression());
            Date date = new Date();
            log.debug("当前时间:" + date);
            Date nextDate = cron.getNextValidTimeAfter(date);//下次执行时间
            log.debug("下次时间:" + nextDate);
            applyPlan.setNextExecTime(nextDate);
        }
        BeanUtils.copyBeanProp(applyPlan, addApplyPlanBO);
        Apply apply = appMap.get(addApplyPlanBO.getAppId()).get(0);
        applyPlan.setAppName(apply.getAppName());
        applyPlan.setAppId(apply.getAppId());
        applyPlan.setAppType(apply.getAppType());
        applyPlan.setAppTypeName(apply.getAppTypeName());
        applyPlan.setManualTime(apply.getManualTime());
        if (null != apply.getSupportParam() && apply.getSupportParam() == 1) {
            applyPlan.setSupportParam(apply.getSupportParam());
            // 获取参数 每一次执行计划的参数都是独立的
            List<ApplyStartBO.RobotParam> paramList = addApplyPlanBO.getResourceLibrarieList().stream()
                    .map(item -> {
                        ApplyStartBO.RobotParam robotParam = new ApplyStartBO.RobotParam();
                        robotParam.setName(item.getResourceName());
                        robotParam.setValue(item.getResourceValue());
                        robotParam.setType(item.getResourceType());
                        return robotParam;
                    }).collect(Collectors.toList());
            resourceLibraryService.saveOrUpdateBatch(addApplyPlanBO.getResourceLibrarieList());
            if (paramList.isEmpty()) {
                throw new ServiceException("该应用需要参数，没有检测到参数！");
            } else {
                // json集合转字符串
                applyPlan.setPlanParams(JSONArray.toJSONString(paramList));
            }
        }
        applyPlan.setDeptId(SecurityUtils.getDeptId());
        // 立即执行才执行
        if (addApplyPlanBO.getExcType().equals("0")) {
            // 只指定优先级和应用和人工用时 机器人等下一个阶段判读是否有空闲
            ApplyStartBO applyStartBO = new ApplyStartBO();
            try {
                applyStartBO.setRobotUuid(applyPlan.getAppId());
                //high 高 middle 中 low 低
                applyStartBO.setPriority("high");
                applyStartBO.setPlanParams(applyPlan.getPlanParams());
                applyStartBO.setManualTime(applyPlan.getManualTime());
                applyStartBO.setPlanName(addApplyPlanBO.getPlanName());
                ApplyPlan retApplyPlan = new ApplyPlan();
                isExec = execPlan(applyStartBO, retApplyPlan);
                applyPlan.setTaskUuid(retApplyPlan.getTaskUuid());
            } catch (Exception ex) {
                log.error("分配机器人异常：" + ex.getMessage());
            }
        }

        if (isExec) {
            applyPlan.setTaskStatus(PlanRunStatus.CREATED.getKey());
        } else {
            applyPlan.setTaskStatus(PlanRunStatus.AWAIT_CREATE.getKey());
        }
        applyPlanList.add(applyPlan);
        return this.saveBatch(applyPlanList) || insert.get();
    }

    @Override
    public boolean update(AddApplyPlanBO addApplyPlanBO) {
        List<ApplyPlan> applyPlanList = new ArrayList<>();
        Map<String, List<Apply>> appMap = applyService.lambdaQuery().in(Apply::getAppId, addApplyPlanBO.getAppId()).list().stream().collect(Collectors.groupingBy(Apply::getAppId));
        ApplyPlan applyPlan = new ApplyPlan();
        BeanUtils.copyBeanProp(applyPlan, addApplyPlanBO);
        Apply apply = appMap.get(addApplyPlanBO.getAppId()).get(0);
        applyPlan.setAppName(apply.getAppName());
        applyPlan.setAppId(apply.getAppId());
        applyPlan.setAppType(apply.getAppType());
        applyPlan.setAppTypeName(apply.getAppTypeName());
        applyPlanList.add(applyPlan);

        return super.updateBatchById(applyPlanList);
    }

    /**
     * 查询应用执行计划管理列表
     *
     * @param applyPlan 应用执行计划管理
     * @return 应用执行计划管理
     */
    @Override
    @DataScope(deptAlias = "plan")
    public List<ApplyPlan> list(ApplyPlan applyPlan) {
        return baseMapper.selectApplyPlanList(applyPlan);
    }
}
