package com.ruoyi.business.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import com.ruoyi.business.domain.Apply;
import com.ruoyi.business.domain.bo.AddBatchResourceLibraryBO;
import com.ruoyi.business.service.IApplyService;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.business.mapper.ResourceLibraryMapper;
import com.ruoyi.business.domain.ResourceLibrary;
import com.ruoyi.business.service.IResourceLibraryService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * 资源库管理Service业务层处理
 *
 * @author LiuChengRan
 * @date 2024-06-17
 */
@Service
public class ResourceLibraryServiceImpl extends ServiceImpl<ResourceLibraryMapper, ResourceLibrary> implements IResourceLibraryService {

    @Autowired
    private IApplyService applyService;

    /**
     * 批量修改参数
     * @param addBatchResourceLibraryBO
     * @return
     */
    @Override
    public boolean updateBatch(AddBatchResourceLibraryBO addBatchResourceLibraryBO) {
        // 参数集合
        List<ResourceLibrary> resourceLibraries = new ArrayList<>();
        addBatchResourceLibraryBO.getList().forEach(e -> {
            ResourceLibrary resourceLibrary = new ResourceLibrary();
            resourceLibrary.setId(e.getId());
            resourceLibrary.setResourceValue(e.getResourceValue());
            resourceLibraries.add(resourceLibrary);
        });

        return super.updateBatchById(resourceLibraries);
    }

    /**
     * 批量添加参数
     *
     * @param addBatchResourceLibraryBO
     */
    @Override
    public boolean addBatch(AddBatchResourceLibraryBO addBatchResourceLibraryBO) {
        Apply apply = Optional.ofNullable(applyService.lambdaQuery().eq(Apply::getAppId, addBatchResourceLibraryBO.getAppId()).one())
                .orElseThrow(() -> new RuntimeException("应用不存在"));
        // 参数集合
        List<ResourceLibrary> resourceLibraries = new ArrayList<>();
        // stream统计对象里重复的字段
        Map<String, Long> mapList = addBatchResourceLibraryBO.getList().stream().collect(Collectors.groupingBy(ResourceLibrary::getResourceName, Collectors.counting()));
        mapList.forEach((k, v) -> {
            if (v > 1) {
                throw new ServiceException(k + "重复，请重新输入");
            }
            if(this.lambdaQuery().eq(ResourceLibrary::getAppId, addBatchResourceLibraryBO.getAppId()).eq(ResourceLibrary::getResourceName,k).count()>0){
                throw new ServiceException(k + "已存在，请重新输入");
            }
        });
        addBatchResourceLibraryBO.getList().forEach(e -> {
            ResourceLibrary resourceLibrary = new ResourceLibrary();
            resourceLibrary.setAppId(apply.getAppId());
            resourceLibrary.setAppName(apply.getAppName());
            resourceLibrary.setDeptId(apply.getDeptId());
            resourceLibrary.setDeptName(apply.getDeptName());
            resourceLibrary.setResource(e.getResource());
            resourceLibrary.setResourceName(e.getResourceName());
            resourceLibrary.setResourceType(e.getResourceType());
            resourceLibrary.setResourceValue(e.getResourceValue());
            resourceLibraries.add(resourceLibrary);
        });

        return super.saveBatch(resourceLibraries);
    }

    /**
     * 查询资源库管理列表
     *
     * @param resourceLibrary 资源库管理
     * @return 资源库管理
     */
    @Override
    public List<ResourceLibrary> list(ResourceLibrary resourceLibrary, List<String> appIdList) {
        // appIdList 转,号分隔字符串
        return baseMapper.selectResourceLibraryList(resourceLibrary, null != appIdList ? String.join(",", appIdList) : null);
    }

    @Override
    public boolean save(ResourceLibrary entity) {
        Apply apply = Optional.ofNullable(applyService.lambdaQuery().eq(Apply::getAppId, entity.getAppId()).one())
                .orElseThrow(() -> new RuntimeException("应用不存在"));
        entity.setAppId(apply.getAppId());
        entity.setAppName(apply.getAppName());
        entity.setDeptId(apply.getDeptId());
        entity.setDeptName(apply.getDeptName());
        return super.save(entity);
    }
}
