package com.ruoyi.business.util;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.ruoyi.business.domain.bo.*;
import com.ruoyi.business.domain.vo.*;
import com.ruoyi.business.yddoman.BaseDTO;
import com.ruoyi.business.yddoman.CreateTokenDTO;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.BeanToMapUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.http.HttpClientUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 影刀请求工具类
 *
 * @author #author#
 */
@Slf4j
public class YinDaoHttpUtils {


    // 请求前缀
    private static final String REQUEST_PREFIX = "https://api.winrobot360.com/oapi";
    // 获取token
    private static final String CREATE_TOKEN = REQUEST_PREFIX + "/token/v2/token/create";
    // 查询机器人列表
    private static final String CLIENT_LIST = REQUEST_PREFIX + "/dispatch/v2/client/list";
    // 查询应用列表
    private static final String APP_LIST = REQUEST_PREFIX + "/app/open/query/list";
    // 启动应用
    private static final String APP_START = REQUEST_PREFIX + "/dispatch/v2/job/start";
    // 停止应用
    private static final String APP_STOP = REQUEST_PREFIX + "/dispatch/v2/job/stop";
    // 重新运行
    private static final String APP_RETRY = REQUEST_PREFIX + "/dispatch/v2/job/retry";
    // 查询应用运行结果
    private static final String QUERY_APP_START_RESULT = REQUEST_PREFIX + "/dispatch/v2/job/query";
    // 查询应用运行日志（异常时可以查询原因）
    private static final String QUERY_APP_START_RESULT_LOG = REQUEST_PREFIX + "/dispatch/v2/job/list";
    // 服务器ip
    private static final String SERVER_URL = "http://192.168.10.214:9000";


    /**
     * 启动应用
     */
    public static JobStartVO appStart(ApplyStartBO applyStartBO) throws IllegalAccessException {
        if (!Objects.isNull(applyStartBO.getPlanParams())) {
            List<ApplyStartBO.RobotParam> robotParams = JSON.parseArray(applyStartBO.getPlanParams(), ApplyStartBO.RobotParam.class);
            List<ApplyStartBO.RobotParam> newParamList = robotParams.stream().peek(e -> {
                if ("file".equals(e.getType())) {
//                    e.setValue(SERVER_URL + e.getValue());
                    e.setType("str");
                    e.setValue(SERVER_URL + e.getValue());
                }
            }).collect(Collectors.toList());
            applyStartBO.setParams(newParamList);
            log.debug("请求的参数：" + applyStartBO.getParams());
        }

        // 实体类转map
        Map<String, Object> data = BeanToMapUtil.convertEntityToMap(applyStartBO);

        BaseDTO baseDTO = sendPost(APP_START, data);
        return JSON.parseObject(baseDTO.getData().toString(), JobStartVO.class);
    }

    /**
     * 停止运行
     */
    public static BaseDTO appStop(JobQueryBO jobQueryBO) throws IllegalAccessException {
        // 实体类转map
        Map<String, Object> data = BeanToMapUtil.convertEntityToMap(jobQueryBO);
        BaseDTO baseDTO = sendPost(APP_STOP, data);
        return baseDTO;
    }

    /**
     * 重新运行
     */
    public static BaseDTO appRetry(JobQueryBO jobQueryBO) throws IllegalAccessException {
        // 实体类转map
        Map<String, Object> data = BeanToMapUtil.convertEntityToMap(jobQueryBO);
        BaseDTO baseDTO = sendPost(APP_RETRY, data);
        return baseDTO;
    }

    /**
     * 查询应用运行结果
     */
    public static JobQueryVO queryAppStartResult(JobQueryBO jobQueryBO) throws IllegalAccessException {
        // 实体类转map
        Map<String, Object> data = BeanToMapUtil.convertEntityToMap(jobQueryBO);
        BaseDTO baseDTO = sendPost(QUERY_APP_START_RESULT, data);
        return JSON.parseObject(baseDTO.getData().toString(), JobQueryVO.class);
    }

    /**
     * 查询应用运行结果
     */
    public static JobQueryLogVO queryAppStartResultLog(JobQueryLogBO jobQueryLogBO) throws IllegalAccessException {
        // 实体类转map
        Map<String, Object> data = BeanToMapUtil.convertEntityToMap(jobQueryLogBO);
        BaseDTO baseDTO = sendPost(QUERY_APP_START_RESULT_LOG, data);
        return JSON.parseObject(baseDTO.getData().toString(), JobQueryLogVO.class);
    }

    /**
     * 查询应用列表
     */
    public static List<ListApplyVO> listApp(ListApplyBO ListApplyBO) throws IllegalAccessException {
        Map<String, Object> data = BeanToMapUtil.convertEntityToMap(ListApplyBO);
        BaseDTO baseDTO = sendPost(APP_LIST, data);
        BaseDTO.Page page = baseDTO.getPage();
        List<ListApplyVO> allList = new ArrayList<>();
        boolean isNext = true;
        while (isNext) {
            List<ListApplyVO> ListApplyVos = JSON.parseArray(baseDTO.getData().toString(), ListApplyVO.class);
            allList.addAll(ListApplyVos);
            data = new HashMap<>();
            data.put("page", page.getPage() + 1);
            data.put("size", ListApplyBO.getSize());
            if (page.getPage() + 1 > page.getPages()) {
                isNext = false;
            } else {
                baseDTO = sendGet(APP_LIST, data);
                page = baseDTO.getPage();
            }
        }
        return allList;
    }

    /**
     * 查询机器人列表
     */
    public static List<ListRebotVO> listRebot(ListRebotBO listRebotBO) throws IllegalAccessException {
        Map<String, Object> data = BeanToMapUtil.convertEntityToMap(listRebotBO);
        BaseDTO baseDTO = sendGet(CLIENT_LIST, data);
        BaseDTO.Page page = baseDTO.getPage();
        List<ListRebotVO> allList = new ArrayList<>();
        boolean isNext = true;
        while (isNext) {
            List<ListRebotVO> listRebotVOS = JSON.parseArray(baseDTO.getData().toString(), ListRebotVO.class);
            allList.addAll(listRebotVOS);
            data = new HashMap<>();
            data.put("page", page.getPage() + 1 + "");
            data.put("size", listRebotBO.getSize().toString());
            if (page.getPage() + 1 > page.getPages()) {
                isNext = false;
            } else {
                baseDTO = sendGet(CLIENT_LIST, data);
                page = baseDTO.getPage();
            }

        }
        return allList;
    }


    /**
     * 统一get请求
     *
     * @param map
     */
    private static BaseDTO sendGet(String url, Map<String, Object> map) {
        Map<String, Object> newMap = new HashMap<>();
        // 写死
        newMap.put("accessKeyId", "hVYUk1yCZ3RX6f9q@platform");
        newMap.put("accessKeySecret", "CKaVmug0TcPYhf1nGUjASyveN2s6dpFR");
        Map<String, Object> headMap = new HashMap<>();
        headMap.put("Authorization", "Bearer " + getToken(newMap));
        headMap.put("Content-Type", "application/json");
        String returnStr = HttpClientUtil.postJson(url, headMap, JSONUtil.toJsonStr(map), "utf-8");
        BaseDTO baseDTO = JSON.parseObject(returnStr, BaseDTO.class);
        if (!baseDTO.getSuccess()) {
            throw new ServiceException("请求影刀接口失败，状态码为" + baseDTO.getCode() + "原因是：" + baseDTO.getMsg());
        }
        return baseDTO;
    }


    /**
     * 统一post请求
     *
     * @param map
     */
    private static BaseDTO sendPost(String url, Map<String, Object> map) {
        Map<String, Object> tokenMap = new HashMap<>();
        // 写死
        tokenMap.put("accessKeyId", "hVYUk1yCZ3RX6f9q@platform");
        tokenMap.put("accessKeySecret", "CKaVmug0TcPYhf1nGUjASyveN2s6dpFR");
        Map<String, Object> headMap = new HashMap<>();
        headMap.put("Accept", "application/json");
        headMap.put("Content-Type", "application/json;charset=utf-8");
        headMap.put("authorization", "Bearer " + getToken(tokenMap));

        log.debug("请求的url：" + url);
        log.debug("请求的参数：" + JSONUtil.toJsonStr(map));
        log.debug("请求头：" + JSONUtil.toJsonStr(headMap));
        String returnStr = HttpClientUtil.postJson(url, headMap, JSONUtil.toJsonStr(map), "utf-8");

        if (StringUtils.isEmpty(returnStr)) {
            throw new ServiceException("请求影刀接口失败");
        }
        BaseDTO baseDTO = JSON.parseObject(returnStr, BaseDTO.class);
        if (!baseDTO.getSuccess()) {
            throw new ServiceException("请求影刀接口失败，状态码为" + baseDTO.getCode() + "原因是：" + baseDTO.getMsg());
        }
        return baseDTO;
    }

    /**
     * 获取token
     *
     * @param map accessKeyId  accessKeySecret
     * @return
     */
    private static String getToken(Map<String, Object> map) {
        String s = HttpClientUtil.getHttp(CREATE_TOKEN, map);
        BaseDTO baseDTO = JSON.parseObject(s, BaseDTO.class);
        if (!baseDTO.getSuccess()) {
            throw new ServiceException("获取影刀token失败！");
        }
        Object data = baseDTO.getData();
        CreateTokenDTO createTokenDTO = JSON.parseObject(data.toString(), CreateTokenDTO.class);
        return createTokenDTO.getAccessToken();
    }

}